## this is the set of imaging code
## Developed at Allegro, ALMA Arc node Leiden 


from wvrScaleAnalysis import *
from wvrScaleTools import *
from wvrScaleImportFuncs import *


## call generate files
def WVRS_generate_files_forBPimage(asdm, verbose = True, overwrite=False, intent=None, outputdir='image', scans_use=False, scale=1.0,flag_ants='', meta_use=None, refant_use='', fluxSet=1.0, specSet=0.0, freqSet = ''):
    ''' Generates all the files for the WVR Image analysis, namely:
    * measurement set (``asdm.ms``)
    * wvr calibration table (``asdm.wvr``)
    * scaled wvr calibration table (``asdm.scl.wvr``)
    Note it DOES remake the ms file, but is kept separate from that for analysis to be stand alone
       
    :param asdm: ASDM filename
    :type asdm: str
    :keyword verbose: verbose
    :type verbose: boolean
    :keyword overwrite: overwrite existing files
    :type overwrite: boolean
    :keyword intent: observing intent of the scans to analyse
    :type intent: str
    :param outputdir: copy the created files into this directory
    :type outputdir: str
    :param scans_use: to specify scans to read in for field/source e.g. '0:3'
    :type scans_use: str
    :param scale: the WVR scaling factor
    :type scale: float
    :param flag_ants: antennas to really flag
    :type flag_ants: str
    :keyword meta_use: metadata information
    :type meta_use: dictionary
    :param refant_use: antenna for the reference ant in calibration
    :type refant_use: str
    :param fluxSet: to set the absolute flux scaling - default is set to 1.0
    :type fluxSet: float
    :param specSet: to set the spectral index if a flux has aslo been specified- default is set to 0.0
    :type specSet: float
    :param freqSet: to set the absolute freq for the flux setting in setjy - default is '' and so will be the middle band of the observations
    :type fluxSet: str
    '''
    print scale,fluxSet,specSet,freqSet

    summary={}
    
    if not os.path.exists(outputdir):
        os.system('mkdir %s' % outputdir)
    
    # filenames
    if outputdir is None:
        vis  = '%s.ms' % asdm.split('/')[-1]
        wvr = '%s.ms.wvr' % asdm.split('/')[-1]
        wvrscl  = '%s.ms.wvrscl' % asdm.split('/')[-1]
        wvrcal = '%s.ms.wvr.cal' % asdm.split('/')[-1]
        wvrsclcal  = '%s.ms.wvrscl.cal' % asdm.split('/')[-1]
        tsys = '%s.ms.tsys' % asdm.split('/')[-1]
        wvrcalpreph = '%s.ms.wvr.cal.ap_pre_bp' % asdm.split('/')[-1]
        wvrsclcalpreph  = '%s.ms.wvrscl.cal.ap_pre_bp' % asdm.split('/')[-1]
        wvrcalbp = '%s.ms.wvr.cal.bp' % asdm.split('/')[-1]
        wvrsclcalbp  = '%s.ms.wvrscl.cal.bl' % asdm.split('/')[-1]
        wvrcalinf = '%s.ms.wvr.cal.phase_inf' % asdm.split('/')[-1]
        wvrsclcalinf  = '%s.ms.wvrscl.cal.phase_inf' % asdm.split('/')[-1]
    else:
        vis = '%s/%s.ms' % (outputdir, asdm.split('/')[-1])
        wvr = '%s/%s.ms.wvr' % (outputdir, asdm.split('/')[-1])
        wvrscl = '%s/%s.ms.wvrscl' % (outputdir, asdm.split('/')[-1])
        wvrcal = '%s/%s.ms.wvr.cal' % (outputdir, asdm.split('/')[-1])
        wvrsclcal = '%s/%s.ms.wvrscl.cal' % (outputdir, asdm.split('/')[-1])
        tsys = '%s/%s.ms.tsys' % (outputdir, asdm.split('/')[-1])
        wvrcalpreph = '%s/%s.ms.wvr.cal.ap_pre_bp' % (outputdir, asdm.split('/')[-1])
        wvrsclcalpreph = '%s/%s.ms.wvrscl.cal.ap_pre_bp' % (outputdir, asdm.split('/')[-1])
        wvrcalbp = '%s/%s.ms.wvr.cal.bp' % (outputdir, asdm.split('/')[-1])
        wvrsclcalbp = '%s/%s.ms.wvrscl.cal.bp' % (outputdir, asdm.split('/')[-1]) 
        wvrcalinf = '%s/%s.ms.wvr.cal.phase_inf' % (outputdir, asdm.split('/')[-1])
        wvrsclcalinf = '%s/%s.ms.wvrscl.cal.phase_inf' % (outputdir, asdm.split('/')[-1])


    # normal
    summary['vis']=vis
    # the wvr solutions
    summary['wvr']=wvr
    summary['wvrscl']=wvrscl
    # the wvr solutions applied with bp, phase inf and flux (corrected col)
    summary['wvrcal']=wvrcal
    summary['wvrsclcal']=wvrsclcal

    if verbose: print " "
    if verbose: print " * Reading in ASDM"
    
    if ( not os.path.exists(vis) ) or ( os.path.exists(vis) and overwrite):
        if intent is None:
            if verbose: print " Reading in BANDPASS of %s" % asdm
            check = asdm_importToCASA(asdm, vis=vis, intent='BANDPASS', verbose=verbose, scans_use=scans_use, Tsysinc=True)
            intent = 'BANDPASS'
            if check is False:
                if verbose: print " no BANDPASS found!"
                #if verbose: print " Reading in PHASE of %s" % asdm
                #if verbose: print " The outputs for WVR will therefore not likely be representative"
                #check = asdm_importToCASA(asdm, vis=vis, intent='PHASE', verbose=verbose, scans_use=scans_use, Tsysinc=True)
                #intent = 'PHASE'
        else:
            print ' ERROR ther is no Bandpass source '
            #check = asdm_importToCASA(asdm, vis=vis, intent=intent, verbose=verbose, scans_use=scans_use,  Tsysinc=True)
            #intent = 'FLUX'
    # resetting the asdm and dataset_type
    else:
        print "   Nothing to do. MS is already there"
    


    if meta_use is None:
        print ' Getting Meta data required'
        meta_use = WVRS_metashort(vis)

    if verbose: print " Doing the Required initial flagging"
    # Flag the data based the standard flags
    flagdata(vis = vis,  mode = 'manual',   autocorr = True,  flagbackup = False)
  
    flagdata(vis = vis,  mode = 'manual', intent = '*POINTING*,*ATMOSPHERE*',flagbackup = False)
   
     
    flagcmd(vis = vis,  inpmode = 'table',  useapplied = True,  action = 'apply')

    

    # Flag the antennas 
    if len(flag_ants) > 1 :
        if verbose: print " Doing the optional antenna flagging"
        flagdata(vis = vis, mode = 'manual', antenna=flag_ants,  flagbackup = False)
    

    # Generate the Tsys
    if verbose: print " Generating the Tsys"
    if ( not os.path.exists(tsys) ) or ( os.path.exists(tsys) and overwrite ):
		os.system('rm -rf %s' % tsys)
		gencal(vis=vis, caltable=tsys, caltype='tsys')


    # Find and Flag the tsys end channels
      
    tb.open(tsys)
    TsysSpws=np.unique(tb.getcol("SPECTRAL_WINDOW_ID"))
    tb.close()

    TsysSpwEdge=[]
    tb.open(tsys+'/SPECTRAL_WINDOW')
    for TsysSpw in TsysSpws:
        widthSpw = tb.getcell("NUM_CHAN",TsysSpw)
        if widthSpw == 64:
            TsysSpwEdge.append(str(TsysSpw)+':0~4;59~63')
        else:
            TsysSpwEdge.append(str(TsysSpw)+':0~7;120~127') 
            
    TsysSpwEdge=','.join(TsysSpwEdge)
    tb.close()

    flagdata(vis =tsys, mode = 'manual',spw = TsysSpwEdge, flagbackup = False)

   

    # if we have a MS, we perform the WVR calibration and apply it - optionally with Tsys - yes for images here
    if verbose: print " * Doing Standard WVR correction, Applying WVR and Tsys"
    WVRcalib(vis, wvr, overwrite=True, tsystab=tsys)
    if verbose: print ''


    # split corrected
    if verbose: print " Splitting out the Standard WVR corrected data"

    
    tb.open(vis+'/SPECTRAL_WINDOW')
    allSpwName = tb.getcol('NAME')
    tb.close()

    msmd.open(vis)
    SpwForBp = msmd.spwsforintent("CALIBRATE_BANDPASS#ON_SOURCE")
    msmd.close()

    spwmapped_int = []
    for SpwSci in SpwForBp:
        if allSpwName[SpwSci][-8:] == 'FULL_RES':
            spwmapped_int.append(SpwSci)


    # have as a list now 
    spwmapped = [str(spwi) for spwi in spwmapped_int]
    spwmapped = ','.join(spwmapped)
    os.system('rm -rf %s' % wvrcal)
    split(vis=vis, outputvis=wvrcal, spw=spwmapped, datacolumn='corrected')


    # additional flags
    if verbose: print " Additional flags of data - Shadowing and Edge Channels "

    flagdata(vis=wvrcal, mode='shadow', flagbackup=False)


    # Find SPW width for edge flagging
    tb.open(wvrcal)
    sciSpws=np.unique(tb.getcol("DATA_DESC_ID"))
    tb.close()

    sciSpwEdge=[]
    tb.open(wvrcal+'/SPECTRAL_WINDOW')
    for sciSpw in sciSpws:
        widthSpw = tb.getcell("NUM_CHAN",sciSpw)
        if widthSpw == 64:
            sciSpwEdge.append(str(sciSpw)+':0~4;59~63')
        elif widthSpw == 128:
            sciSpwEdge.append(str(sciSpw)+':0~7;120~127') 
            
    sciSpwEdge=','.join(sciSpwEdge)
    tb.close()

    if len(sciSpwEdge) >  1:
        flagdata(vis=wvrcal, mode='manual', spw=sciSpwEdge, flagbackup=False)

    # setjy to normalise the flux if a valve not input
    print ""
    print " NOTE: setjy to will give a NORMALISED flux for the BP calibrator "
    print " NOTE: unless you have specified an input flux an index"
    print " NOTE: the flux will be %s Jy and a spectral index of %s "%(fluxSet,specSet)
    
    
    if len(freqSet) < 2:
        freqSet = str(meta_use['ref_freq'])+'Hz'
   

    setjy(vis=wvrcal, standard='manual', field='0', fluxdensity = [fluxSet,0,0,0], spix = specSet, reffreq = freqSet)
    # freq is irrelavant in default mode - flat index 

    # bandpass
    if verbose: print " Running the Bandpass solution "

    ## need to pic the centre of the band for the phases to use
    ## find where the x and y are minimized
    if len(refant_use) < 2:
        antR = np.sqrt(meta_use['ant']['x']**2 + meta_use['ant']['y']**2)
        minantR_ID = np.where(antR == np.min(antR))[0][0]
        refant_use = meta_use['ant']['name'][minantR_ID]

    if verbose: print ''
    if verbose: print ' Will use reference antenna %s'%refant_use
    if verbose: print ''

    ## find centre of bands for inital phase up
    ## EVEN if a narrow channel - should be fine for BP source 
    ## later version to add freq width dependancy 
    sciSpwBp=[]
    tb.open(wvrcal+'/SPECTRAL_WINDOW')
    for sciSpw in sciSpws:
        widthSpw = tb.getcell("NUM_CHAN",sciSpw)
        rangespw = str(sciSpw)+':'+str(widthSpw/2-widthSpw/6)+'~'+str(widthSpw/2+widthSpw/6)
        sciSpwBp.append(rangespw)
       
            
    sciSpwBp=','.join(sciSpwBp)
    tb.close()

    ## BP solns will ALWAYS be redone
    os.system('rm -rf %s' %wvrcalpreph)
        
    gaincal(vis=wvrcal, caltable=wvrcalpreph,refant=refant_use,calmode='p',spw=sciSpwBp, solint='int')

    os.system('rm -rf %s' %wvrcalbp)
    bandpass(vis=wvrcal, caltable=wvrcalbp, gaintable=wvrcalpreph,combine='scan', refant=refant_use, solint='inf', bandtype='B', solnorm=False) # let it carry the flux scaling as phase-int applied already


    # now apply bp and solve for phase with inf - one offset solution
    # if WVR scaling in 'better' the phases should be more stable from the WVR scaled solution aplication below
    os.system('rm -rf %s' %wvrcalinf)
    gaincal(vis=wvrcal, caltable=wvrcalinf,refant=refant_use,gaintable=wvrcalbp,calmode='p',spw=sciSpwBp, solint='inf', combine='scan')
   
    # and the application
    applycal(vis=wvrcal, gaintable=[wvrcalbp,wvrcalinf],flagbackup=False,calwt=True)



    ############################# WVR SCALING VERSION  ##########
    if verbose: print " * Doing Scaled WVR correction, Applying scaled WVR and Tsys"
    if scale == 1.0:
              print ' WARNING - the scaling factor is not changed from 1.0'
              print ' WARNING - the output images will be no different'
    else:
        print ''
        print ' Using the WVR scaling factor %s'%scale

    WVRcalib(vis, wvrscl, overwrite=True, scale=scale, tsystab=tsys)
    if verbose: print ''
    
          
    # split corrected
    if verbose: print " Splitting out the Standard WVR corrected data"

        
    # found spwmapped earlier - same dataset - different WVR applied (with scaling)
    os.system('rm -rf %s' % wvrsclcal)
    split(vis=vis, outputvis=wvrsclcal, spw=spwmapped, datacolumn='corrected')


    # additional flags
    if verbose: print " Additional flags of data - Shadowing and Edge Channels "

    flagdata(vis=wvrsclcal, mode='shadow', flagbackup=False)


    if len(sciSpwEdge) >  1:
        flagdata(vis=wvrsclcal, mode='manual', spw=sciSpwEdge, flagbackup=False)

    # setjy to normalise the flux
    print " NOTE: setjy to will give a NORMALISED flux for the BP calibrator "
    print " NOTE: the flux will be 1.0 Jy and a spectral index of 0.0 "
    print " NOTE: which is suitable for an image comparion "

    freq_ref = str(meta_use['ref_freq'])+'Hz'
    setjy(vis=wvrsclcal, standard='manual', field='0', fluxdensity = [1,0,0,0], spix = 0.0, reffreq = freq_ref)
    # freq should be irrelavent 

    # bandpass
    if verbose: print " Running the Bandpass solution "

    
    
    ## BP solns will ALWAYS be redone
    os.system('rm -rf %s' %wvrsclcalpreph)
        
    gaincal(vis=wvrsclcal, caltable=wvrsclcalpreph,refant=refant_use,calmode='p',spw=sciSpwBp, solint='int')

    os.system('rm -rf %s' %wvrsclcalbp)
    bandpass(vis=wvrsclcal, caltable=wvrsclcalbp, gaintable=wvrsclcalpreph, refant=refant_use, combine='scan', solint='inf', bandtype='B', solnorm=False) # let it carry the flux scaling as phase-int applied already


    # now apply bp and solve for phase with inf - one offset solution
    # if WVR scaling in 'better' the phases should be more stable from the WVR scaled solution aplication below
    os.system('rm -rf %s' %wvrsclcalinf)
    gaincal(vis=wvrsclcal, caltable=wvrsclcalinf,refant=refant_use,gaintable=wvrsclcalbp,calmode='p',spw=sciSpwBp, solint='inf', combine='scan')
   
    # and the application
    applycal(vis=wvrsclcal, gaintable=[wvrsclcalbp,wvrsclcalinf],flagbackup=False,calwt=True)


    return summary  # outputs the file names - use the data with solns applied 




def makeBPimages(toClean, verbose=True, overwrite=False, outputdir='image',freq_obs=None, band_obs=None,cell_size=None, image_size=None,max_bl=None, meta_use=None, nits=None, fluxSet = 1.0):
    ''' This will real in the data sets with WVR calibrations applied
    and will proceed to image them with either the inputs of what is found 
    from the meta data 
       
    :param toClean: calibrated ms filename
    :type toClean: str
    :keyword verbose: verbose
    :type verbose: boolean
    :keyword overwrite: overwrite existing files
    :type overwrite: boolean
    :param outputdir: the output directory hard coded as the image one
    :type outputdir: str
    :param freq_obs: frequency of the observations in Hz
    :type freq_obs: float
    :param band_obs: observing band
    :type band_obs: int
    :param cell_size: cell size to use in the cleaning (in arcseconds)
    :type cell_size: float 
    :param image_size: size of the image in pixels
    :type image_size: int
    :param max_bl: maximum baseline length to use for cell size (if not input)
    :type max_bl: float
    :param meta_use: the metadata of the observations
    :type meta_data: dictionary
    :param nits: number of iterations for cleaning (optional)
    :type nits: int
    :param fluxSet: to set the absolute flux scaling - default is set to 1.0
    :type fluxSet: float
    '''
    

    # filenames - the ms files are input
    if outputdir is None:
        inputvis  = '%s' % toClean.split('/')[-1]
        outputimg = '%s.bpimage' % toClean.split('/')[-1]
       
    else:
        inputvis = '%s/%s' % (outputdir,toClean.split('/')[-1])
        outputimg = '%s/%s.bpimage' % (outputdir, toClean.split('/')[-1])
        


    if band_obs is None or max_bl is None:
        if meta_use is None:
            print ' Meta data not supplied - Getting Meta data required'
            meta_use = WVRS_metashort(toClean)
        freq_obs = meta_use['ref_freq']
        band_obs = meta_use['band']
        # use the projected baseline length
        max_bl = np.max(meta_use['bl']['uv'])


    if cell_size is None:
        # based on freq and projected baselines length resolution / 5 
        cell_size = 206265. * ( 1.2 * 2.9979e8 ) / ( 5. * max_bl * freq_obs)
        cell_size_im = str(cell_size)[0:6]+'arcsec'
    else:
        cell_size_im = str(cell_size)[0:6]+'arcsec'


    if image_size is None:
        # bands 1 to 10 are index 0 to 9 e.g. band 7 is index 6
        pbsize = np.array([0,0,62.9,41.9,30.9,27.3,17.8,13.4,9.6,8.2])
        pb_use = pbsize[band_obs - 1]
        image_size = pb_use/cell_size
        if image_size > 1024:
            image_size = 1024
            centre_mask = 'circle[[512pix, 512pix], 15pix]'
            outer_mask = 'annulus[[512pix, 512pix], [200pix,500pix]]' 

        elif image_size > 512 and image_size < 1024:
            image_size = 512
            centre_mask = 'circle[[256pix, 256pix], 15pix]'
            outer_mask = 'annulus[[256pix, 256pix], [200pix,250pix]]'
        else:
            image_size = 256
            centre_mask = 'circle[[128pix, 128pix], 15pix]'
            outer_mask = 'annulus[[128pix, 128pix], [200pix,120pix]]'
         # not the full PB but for long baselines this can be too large
         # past 5000 - so the centre is suffice for the BP - point source

    if nits is None:
        nits = 50

    # now run the clean
    os.system('rm -rf %s*' % outputimg) 
    clean(vis=inputvis, imagename=outputimg, cell=cell_size_im, imsize=image_size, niter=nits, mode='mfs', mask=centre_mask)
    exportfits(imagename=outputimg+'.image', fitsimage=outputimg+'.fits')        
   

    # now run the analysis - region values and then the noise
    imagefit_cent = imstat(imagename=outputimg+'.image', region=centre_mask)
    imagefit_outer = imstat(imagename=outputimg+'.image', region=outer_mask)

    if verbose: print ''
    if verbose: print ' Comparing to the expect source flux %s Jy '%fluxSet
    if verbose: print ''
    # parse the info and the return it 
    img_output={}
    img_output['peak']=imagefit_cent['max'][0]  # in Jy/bm
    img_output['rms']=imagefit_outer['rms'][0]*1000. # in mJy/bm
    img_output['coh']=(imagefit_cent['max'][0]/fluxSet)*100. # comopare to the expected peak flux - default is normalised to 1.0 Jy 
    img_output['snr']=imagefit_cent['max'][0]/imagefit_outer['rms'][0]
    return img_output



def makeBPpng(imageBase, statsImg=None, verbose = True,  outputdir='image', fluxSet=1.0):
    ''' This will take the images and make the png and 
    report the peak and the rms from the region 


    :param imageBase: the asdm name is input 
    :type imageBase: str
    :param statsImg: these are the statistics to read and plot on the image
    :type statsImg: list of two dictionaries
    :keyword verbose: to print all the extra statements
    :type verbose: boolean
    :param outputdir: the output directory to store everything
    :type outputdir: str
    :param fluxSet: the flux peak of the bandpass source that can be set (default=1.0 Jy - normalised)
    :type fluxSet: float 

    '''


    # directories

    # filenames
    if outputdir is None:
        inputstd = '%s.ms.wvr.cal.bpimage.fits' % imageBase.split('/')[-1]
        inputscl = '%s.ms.wvrscl.cal.bpimage.fits' % imageBase.split('/')[-1]
       
    else:
        inputstd = '%s/%s.ms.wvr.cal.bpimage.fits' % (outputdir,imageBase.split('/')[-1])
        inputscl = '%s/%s.ms.wvrscl.cal.bpimage.fits' % (outputdir, imageBase.split('/')[-1])
        
    
    # set up the figure
    pl.rcParams['ytick.major.pad'] = 12
    pl.rcParams['xtick.major.pad'] = 12
    
    pl.rcParams['xtick.direction'] = 'in'
    pl.rcParams['ytick.direction'] = 'in'
    
    pl.rcParams['xtick.major.size'] = 10
    pl.rcParams['xtick.minor.size'] = 5
    pl.rcParams['ytick.major.size'] = 10
    pl.rcParams['ytick.minor.size'] = 5
    
    pl.rcParams['ytick.labelsize'] = 12
    pl.rcParams['xtick.labelsize'] = 12
    
    figsize=[10,5]
    figsize=np.array(figsize)
    
    plt.close(1)
    
    
    fig=plt.figure(1, figsize=figsize)
    fig.clf()
    
    if fluxSet == 1.0:
        fig.suptitle('Normalised Bandpass Source Images - The flux is set to be 1 Jy')
    else:
        fig.suptitle('Bandpass Source Images - The flux was set manually at %s Jy'%fluxSet)

    ax1img=fig.add_subplot(1,2,1)
    ax2img=fig.add_subplot(1,2,2)
    
    # need to check if files exist - should do ...
    if os.path.exists(inputstd):
        fits_std = inputstd
    else:
        print ' WARNING - There is no fits image - please check the scripts run correctly '
    
    if os.path.exists(inputscl):
        fits_scl = inputscl
    else:
        print ' WARNING - There is no fits image - please check the scripts run correctly '
    
    headImg = fits.getheader(fits_std)


    nx    = headImg['NAXIS1']
    xpix  = headImg['CRPIX1']
    xdelt = headImg['CDELT1']
    
    ny    = headImg['NAXIS2']
    ypix  = headImg['CRPIX2']
    ydelt = headImg['CDELT2']
    
    # Convert from degrees to arcsecs
    xval = ((np.arange(nx) - xpix + 1) * xdelt)*3600
    yval = ((np.arange(ny) - ypix + 1) * ydelt)*3600
    
    # Create dictionaries containing axes and images
    img_std = {'image':np.squeeze(fits.getdata(fits_std)), 'ra_offset':xval, 'dec_offset':yval}
    img_scl = {'image':np.squeeze(fits.getdata(fits_scl)), 'ra_offset':xval, 'dec_offset':yval}


    ax1img.set_xlabel('Right Ascension Offset (arcsec)', labelpad=10,fontsize=14)
    ax2img.set_xlabel('Right Ascension Offset (arcsec)', labelpad=10,fontsize=14)
    ax1img.set_ylabel(' Declination Offset (arcsec)', labelpad=10,fontsize=14)
    ax1img.set_title(' WVR Standard Application  ', fontsize=14)
    ax2img.set_title(' WVR Scaled Application  ', fontsize=14)

    ax1img.grid(False)
    ax2img.grid(False)

    # set the extent - use the xval and yval divided by 10 say
    xmin = np.min(xval)/10.
    xmax = np.max(xval)/10.
    ymin = np.min(yval)/10.
    ymax = np.max(yval)/10.

    ax1img.set_xlim(xmax,xmin)
    ax1img.set_ylim(ymin,ymax)
    ax2img.set_xlim(xmax,xmin)
    ax2img.set_ylim(ymin,ymax)

    # Set physical range of colour map
    cxmin = img_std['ra_offset'][0]
    cxmax = img_std['ra_offset'][-1]
    cymin = img_std['dec_offset'][0]
    cymax = img_std['dec_offset'][-1]

    # Set limits and tics of colorbar 
    cbmin = 0.00001
    cbmax = np.max([np.max(img_std['image']), np.max(img_scl['image'])])
    cbtmj = 0.01
    cbtmn = 0.001
    cbnum = int((cbmax-cbmin)/cbtmn)

    # Plot continuum image as a colour map
    colmap1 = ax1img.imshow(img_std['image'][::-1],
	extent=[cxmin,cxmax,cymin,cymax],
       	cmap='cubehelix_r')

    colmap2 = ax2img.imshow(img_scl['image'][::-1],
	extent=[cxmin,cxmax,cymin,cymax],
        cmap='cubehelix_r')


    # annotations if input  - EDIT 
    if statsImg:
        ax1img.annotate('Peak = '+str(statsImg[0]['peak'])[0:5]+' Jy/bm, Coherence = '+str(statsImg[0]['coh'])[0:4]+' %' ,xy=(0,0), xytext=(0.065,0.12), size = 10, color = 'k',backgroundcolor = 'w', textcoords='axes fraction')
        ax1img.annotate('RMS = '+str(statsImg[0]['rms'])[0:5]+' mJy/bm, SNR ='+str(statsImg[0]['snr'])[0:5] ,xy=(0,0), xytext=(0.065,0.07), size = 10, color = 'k',backgroundcolor = 'w', textcoords='axes fraction')
       

        ax2img.annotate('Peak = '+str(statsImg[1]['peak'])[0:5]+' Jy/bm, Coherence = '+str(statsImg[1]['coh'])[0:4]+' %' ,xy=(0,0), xytext=(0.065,0.12), size = 10, color = 'k',backgroundcolor = 'w', textcoords='axes fraction')
        ax2img.annotate('RMS = '+str(statsImg[1]['rms'])[0:5]+' mJy/bm, SNR ='+str(statsImg[1]['snr'])[0:5] ,xy=(0,0), xytext=(0.065,0.07), size = 10, color = 'k',backgroundcolor = 'w', textcoords='axes fraction')

    # Output the png in the output directory
    fig.savefig(inputstd+'_bandpass_comp.png', dpi=300.)
